from sklearn.linear_model import LinearRegression as LR
from .RegressionModel import RegressionModel


class LinearRegression(RegressionModel):
    def __init__(self, **kwargs) -> None:
        super().__init__(**kwargs)

        if not self._fileload:
            self._model = LR()



from sklearn.linear_model import LogisticRegression as LR
from .ClassificationModel import ClassificationModel


class LogisticRegression(ClassificationModel):
    def __init__(self, **kwargs):
        super().__init__(**kwargs)
        self._model = LR(max_iter=99999999)

    def _check_bin_cls(self, ds):
        if len(ds.unique()) == 2:
            return True

        return False

    def _feature_init(self, **kwargs):
        if isinstance(kwargs["target"], str):
            if not self._check_bin_cls(self._dataf[kwargs["target"]]):
                raise ValueError("Multi Class classification is not Supported Yet")

        return super()._feature_init(**kwargs)

    def _custom_train_test_init(self, **kwargs):
        if not self._check_bin_cls(kwargs["y_Train"]):
            raise ValueError("Multi Class Classification is not Supported Yet")

        return super()._custom_train_test_init(**kwargs)



from .ClassificationModel import ClassificationModel
from sklearn.tree import DecisionTreeClassifier, plot_tree
import matplotlib.pyplot as plt
import numpy as np


class DecisionTreeModel(ClassificationModel):
    def __init__(self, **kwargs) -> None:
        super().__init__(**kwargs)
        self._citeration = kwargs.get("citeration", "gini")
        self._max_depth = kwargs.get("max_depth", None)
        self._max_features = kwargs.get("max_features", None)
        self._min_sample_split = kwargs.get("min_sample_split", 2)
        self._min_samples_leaf = kwargs.get("min_samples_leaf", 1)
        self._max_leaf_nodes = kwargs.get("max_leaf_nodes", None)

        self._model = DecisionTreeClassifier(
            criterion=self._citeration,
            max_depth=self._max_depth,
            max_features=self._max_features,
            min_samples_split=self._min_sample_split,
            min_samples_leaf=self._min_samples_leaf,
            max_leaf_nodes=self._max_leaf_nodes,
            random_state=42,
        )

    def test(self, X_test=None, Y_test=None, plot_show=False):
        ret = super().test(X_test, Y_test, plot_show=False)

        if plot_show:
            self.tree_plot()

        return ret

    def tree_plot(self):
        if self._max_depth and self._max_depth < 20:
            plot_tree(self._model)
            plt.show()
        else:
            print("Tree size too huge to show.\n Try Lowering the max_depth attribute")



from sklearn.linear_model import Ridge
from .RegressionModel import RegressionModel


class RidgeRegression(RegressionModel):
    def __init__(self, **kwargs) -> None:
        super().__init__(**kwargs)

        if not self._fileload:
            if "ridge" in kwargs:
                if isinstance(kwargs["ridge"], float):
                    self._model = Ridge(alpha=kwargs["ridge"])
                else:
                    raise ValueError("Alpha value of ridge should be a float")
            else:
                self._model = Ridge(alpha=0.0)



import numpy as np
import matplotlib.pyplot as plt
from sklearn.metrics import (
    mean_absolute_error,
    mean_absolute_percentage_error,
    mean_squared_error,
    r2_score,
)

from .BaseModel import BaseModel


class RegressionModel(BaseModel):
    def __init__(self, **kwargs) -> None:
        super().__init__(**kwargs)

    def test(self, X_test=None, Y_test=None, plot_show=False):
        if not X_test and not Y_test:
            X_test = self._xtest
            Y_test = self._ytest
        else:
            if self._poly is not None:
                X_test = self._poly.transform(X_test)

        if X_test is not None and Y_test is not None:
            yPred = self._model.predict(X_test)

            merr = mean_squared_error(Y_test, yPred)
            rmerr = np.sqrt(merr)
            r2 = r2_score(Y_test, yPred)
            mae = mean_absolute_error(Y_test, yPred)
            mape = mean_absolute_percentage_error(Y_test, yPred)

            self._perf_metrics["MeanSqERR"] = merr
            self._perf_metrics["RootMeanSqERR"] = rmerr
            self._perf_metrics["MeanAbsERR"] = mae
            self._perf_metrics["MeanAbsPercERR"] = mape
            self._perf_metrics["r2"] = r2

            fig, axs = plt.subplots()
            axs.scatter(Y_test, yPred)
            axs.set_xlabel("True Values")
            axs.set_ylabel("Predictions")
            axs.set_title("True vs Predicted Prices")
            axs.grid(True)

            scatter_graph = fig

            if plot_show:
                plt.show()

            self._cache["scatter_plot"] = scatter_graph

            return yPred, merr, r2, scatter_graph
        else:
            if not X_test and not Y_test:
                raise ValueError("No Test Data Available")
            elif not X_test:
                raise ValueError("No X_Test Data Available")
            raise ValueError("No Y_test Data Available")

    def perf_print(self):
        print(f"R2 score: {self._perf_metrics.get("r2","NA")}")
        print(f"Mean Absolute Error: {self._perf_metrics.get("MeanAbsERR","NA")}")
        print(
            f"Mean Absolute Perc Error: {self._perf_metrics.get("MeanAbsPercERR","NA")}"
        )
        print(f"Mean Squared Error: {self._perf_metrics.get("MeanSqERR","NA")}")
        print(
            f"Root Mean Squared Error: {self._perf_metrics.get("RootMeanSqERR","NA")}\n\n"
        )

    def PredscatterPlot(self, **kwargs):
        plot = self._cache.get("scatter_plot", None)
        if kwargs.get("force", False) or not plot:
            self.train()
            plot = self._cache.get("scatter_plot", None)
            if not plot:
                raise ValueError("Something Wrong INternally")
        if "filename" in kwargs:
            filename = kwargs.get("filename", "scatter_plot")
            filename = filename + ".png"
            plot.savefig(filename)

        if kwargs.get("show", True):
            plt.show(plot)

        return plot

    @property
    def r2_score(self):
        return self._perf_metrics.get("r2", None)

    @property
    def mean_sq_err(self):
        return self._perf_metrics.get("MeanSqERR", None)

    @property
    def root_mean_sq_err(self):
        return self._perf_metrics.get("RootMeanSqERR", None)

    @property
    def mean_abs_err(self):
        return self._perf_metrics.get("MeanAbsERR", None)

    @property
    def mean_abs_perc_err(self):
        return self._perf_metrics.get("MeanAbsPercERR", None)



import joblib
from sklearn.model_selection import train_test_split
from sklearn.preprocessing import PolynomialFeatures, StandardScaler


class BaseModel:
    def __init__(self, **kwargs) -> None:
        self._model = None

        self._xtrain = None
        self._ytrain = None

        self._xtest = None
        self._ytest = None

        self._dataf = None
        self._X = None
        self._Y = None

        self._poly = None
        self._scaler = None

        self._trained = False
        self._fileload = False

        self._test_split_ratio = 0.2
        self._perf_metrics = {}
        self._cache = {}

        if "data" in kwargs:
            self._dataf = kwargs["data"]

            if "features" in kwargs and "target" in kwargs:
                self._feature_init(**kwargs)

        elif "X_Train" in kwargs and "y_Train" in kwargs:
            self._custom_train_test_init(**kwargs)

        elif "model_file" in kwargs:
            self._model = joblib.load(kwargs["model_file"])
            self._fileload = True

        else:
            raise ValueError("No Params Passed To Initialize")

    def train(self):
        self._model_check()

        if self._xtrain is not None and self._ytrain is not None:
            self._model.fit(self._xtrain, self._ytrain)
            self._trained = True

        elif not self._fileload:
            raise ValueError("Training Data Not Provided")

    def predict(self, X):
        self._model_check()

        if self._scaler:
            X = self._scaler.transform(X)

        if self._poly:
            X = self._poly.transform(X)

        return self._model.predict(X)

    def save_raw(self, filename=None):
        self._model_check()

        if not filename:
            filename = "regression_model"

        filename = filename + ".pkl"
        joblib.dump(self._model, filename)

    def save(self, filename=None):
        if not filename:
            filename = "model"

        filename = filename + ".pkl"

    @classmethod
    def load(cls, filename):
        loaded_instance = joblib.load(filename)
        return loaded_instance

    @property
    def features(self):
        return list(self._X)

    @property
    def target(self):
        return self._Y.name

    @property
    def intercept(self):
        return self._model.intercept_

    @property
    def coeffs(self):
        return self._model.coef_

    def _feature_init(self, **kwargs):
        if isinstance(kwargs["features"], list):
            self._X = self._dataf[kwargs["features"]]
        else:
            raise ValueError("Features Should be a List")
        if isinstance(kwargs["target"], str):
            self._Y = self._dataf[kwargs["target"]]
        else:
            raise ValueError("Target should be single and of type str")

        if kwargs.get("test", False):
            if "test_split_ratio" in kwargs:
                self._test_split_ratio = kwargs.get("test_split_ratio", 0.2)

            self._xtrain, self._xtest, self._ytrain, self._ytest = train_test_split(
                self._X,
                self._Y,
                test_size=self._test_split_ratio,
                random_state=42,
            )

        else:
            self._xtrain = self._X
            self._ytrain = self._Y

        if kwargs.get("scale", False):
            self._scaler_init()

        if kwargs.get("polynomial_regression", False):
            self._poly_init()

    def _custom_train_test_init(self, **kwargs):
        self._xtrain = kwargs["X_Train"]
        self._ytrain = kwargs["y_Train"]

        if kwargs.get("test", False) and "X_Test" in kwargs and "y_Test" in kwargs:
            self._xtest = kwargs["X_Test"]
            self._ytest = kwargs["y_Test"]

    def _poly_init(self):
        self._poly = PolynomialFeatures(degree=2)
        self._xtrain = self._poly.fit_transform(self._xtrain)
        if self._xtest is not None:
            self._xtest = self._poly.transform(self._xtest)

    def _scaler_init(self):
        self._scaler = StandardScaler()
        self._xtrain = self._scaler.fit_transform(self._xtrain)
        if self._xtest is not None:
            self._xtest = self._scaler.transform(self._xtest)

    def _model_check(self):
        if not self._model:
            raise ValueError("No Model Loaded/Initailized")



from setuptools import setup, find_packages

setup(
    name="scikit-milap",
    version="0.1.0",
    description="A wrapper around scikit learn ML models that enable you to create and use models easily.",
    long_description=open("README.md").read(),
    long_description_content_type="text/markdown",
    author="Bhaumik Talwar",
    author_email="bhaumik303@gmail.com",
    url="https://github.com/BhaumikTalwar/SCIKIT-MILAP",
    packages=find_packages(),
    install_requires=[
        "numpy==2.0.1",
        "pandas==2.2.2",
        "matplotlib==3.9.2",
        "scikit-learn==1.5.1",
    ],
    classifiers=[
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.9",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
        "Development Status :: 3 - Alpha",
        "Intended Audience :: Developers",
        "Topic :: Software Development :: Libraries :: Python Modules",
    ],
    python_requires=">=3.9",
)



import matplotlib.pyplot as plt
from sklearn.metrics._plot.confusion_matrix import ConfusionMatrixDisplay
from sklearn.metrics import (
    accuracy_score,
    classification_report,
    confusion_matrix,
    f1_score,
    precision_recall_curve,
    precision_score,
    recall_score,
    roc_auc_score,
    roc_curve,
)

from .BaseModel import BaseModel


class ClassificationModel(BaseModel):
    def __init__(self, **kwargs) -> None:
        super().__init__(**kwargs)

    def test(self, X_test=None, Y_test=None, plot_show=False):
        if not self._trained:
            raise ValueError("Model Not tained Yet")

        if not X_test and not Y_test:
            X_test = self._xtest
            Y_test = self._ytest
        else:
            if self._poly is not None:
                X_test = self._poly.transform(X_test)

        if X_test is not None and Y_test is not None:
            yPred = self._model.predict(X_test)
            yPred_proba = self._model.predict_proba(X_test)

            self._perf_metrics["accuracy"] = accuracy_score(Y_test, yPred)
            self._perf_metrics["precision"] = precision_score(Y_test, yPred)
            self._perf_metrics["recall"] = recall_score(Y_test, yPred)
            self._perf_metrics["f1"] = f1_score(Y_test, yPred)
            self._perf_metrics["roc_auc"] = roc_auc_score(Y_test, yPred)
            self._perf_metrics["conf_matrix"] = confusion_matrix(Y_test, yPred)
            self._perf_metrics["cls_report"] = classification_report(Y_test, yPred)

            yPred_proba = yPred_proba[:, 1]
            fpr, tpr, thresholds = roc_curve(Y_test, yPred_proba)

            fig, axs = plt.subplots()
            axs.plot(
                fpr, tpr, label=f"ROC -CURVE (area ={self._perf_metrics["roc_auc"]})"
            )
            axs.set_xlabel("False Positive Rate")
            axs.set_ylabel("True Positive Rate")
            axs.grid(True)

            self._cache["roc_graph"] = fig

            if plot_show:
                fig.show()
                plt.show()

            self._cache["yPred"] = yPred
            self._cache["yPred_proba"] = yPred_proba

            return (
                yPred,
                self._perf_metrics["accuracy"],
                self._perf_metrics["precision"],
            )
        else:
            if not X_test and not Y_test:
                raise ValueError("No Test Data Available")
            elif not X_test:
                raise ValueError("No X_Test Data Available")
            raise ValueError("No Y_test Data Available")

    def perf_print(self):
        print(f"Accuracy: {self._perf_metrics.get("accuracy","NA")}")
        print(f"Precision: {self._perf_metrics.get("precision","NA")}")
        print(f"Recall Score: {self._perf_metrics.get("recall","NA")}\n\n")
        print(f"F1 Score: {self._perf_metrics.get("f1","NA")}")
        print(f"ROC-AUC Score: {self._perf_metrics.get("roc_auc","NA")}")
        print(f"Confusion Matrix: {self._perf_metrics.get("conf_matrix","NA")}")
        print(f"Classification Report: {self._perf_metrics.get("cls_report","NA")}")

    def roc_graph(self, **kwargs):
        plot = self._cache.get("roc_graph", None)
        if kwargs.get("force", False) or not plot:
            self.train()
            self.test()
            plot = self._cache.get("roc_graph", None)
            if not plot:
                raise ValueError("Something Wrong INternally")
        if "filename" in kwargs:
            filename = kwargs.get("filename", "roc_graph")
            filename = filename + ".png"
            plot.savefig(filename)

        if kwargs.get("show", True):
            plot.show()
            plt.show()

        return plot

    def precision_recall_plot(self, **kwargs):
        if kwargs.get("force", False):
            self.train()
            self.test()
        elif ("pre_recall_plot" in self._cache) and (
            self._cache["pre_recall_plot"] is not None
        ):
            plot = self._cache["pre_recall_plot"]
            plot.show()
            plt.show()

            return plot

        yPred_proba = self._cache["yPred_proba"]
        precision, recall, thresholds = precision_recall_curve(self._ytest, yPred_proba)

        fig, axs = plt.subplots()
        axs.plot(recall, precision, color="blue", lw=2)
        axs.set_xlabel("Recall")
        axs.set_ylabel("Precision")
        axs.grid(True)

        self._cache["pre_recall_plot"] = fig
        plt.show()
        return fig

    def confusion_display(self, **kwargs):
        if kwargs.get("force", False):
            self.train()
            self.test()

        if "conf_disp" in self._cache and self._cache["conf_disp"] is not None:
            disp = self._cache["conf_disp"]
            disp.plot(cmap=plt.cm.Blues)
            plt.show()
            return disp

        cm = None

        if "conf_matrix" in self._cache:
            cm = self._cache["conf_matrix"]
        elif "yPred" in self._cache:
            cm = confusion_matrix(self._ytest, self._cache["yPred"])
        else:
            yPred, _, _ = self.test()
            cm = confusion_matrix(self._ytest, yPred)

        disp = ConfusionMatrixDisplay(confusion_matrix=cm)
        disp.plot(cmap=plt.cm.Blues)
        plt.show()

        self._cache["conf_disp"] = disp
        return disp

    @property
    def accuracy(self):
        return self._perf_metrics.get("accuracy", None)

    @property
    def precision(self):
        return self._perf_metrics.get("precision", None)

    @property
    def recall(self):
        return self._perf_metrics.get("recall", None)

    @property
    def f1(self):
        return self._perf_metrics.get("f1", None)

    @property
    def roc_auc(self):
        return self._perf_metrics.get("roc_auc", None)

    @property
    def conf_matrix(self):
        return self._perf_metrics.get("conf_matrix", None)

    @property
    def cls_report(self):
        return self._perf_metrics.get("cls_report", None)
